@@ -1,5 +1,7 @@
+import itertools
 import json
 import logging
+import os
 import re
 import shlex
 import subprocess
@@ -8,6 +10,7 @@ from typing import Dict, List, Optional, Tuple, Union
 from localstack import config
 from localstack.utils.common import safe_run
 from localstack.utils.container_utils.container_client import (
+    AccessDenied,
     ContainerClient,
     ContainerException,
     DockerContainerStatus,
@@ -16,6 +19,7 @@ from localstack.utils.container_utils.container_client import (
     NoSuchNetwork,
     NoSuchObject,
     PortMappings,
+    RegistryConnectionError,
     SimpleVolumeBind,
     Util,
 )
@@ -87,6 +91,20 @@ class CmdDockerClient(ContainerClient):
                     "Docker process returned with errorcode %s" % e.returncode, e.stdout, e.stderr
                 )
 
+    def unpause_container(self, container_name: str) -> None:
+        cmd = self._docker_cmd()
+        cmd += ["unpause", container_name]
+        LOG.debug("Unpausing container with cmd %s", cmd)
+        try:
+            safe_run(cmd)
+        except subprocess.CalledProcessError as e:
+            if "No such container" in to_str(e.stdout):
+                raise NoSuchContainer(container_name, stdout=e.stdout, stderr=e.stderr)
+            else:
+                raise ContainerException(
+                    "Docker process returned with errorcode %s" % e.returncode, e.stdout, e.stderr
+                )
+
     def remove_image(self, image: str, force: bool = True) -> None:
         cmd = self._docker_cmd()
         cmd += ["rmi", image]
@@ -188,7 +206,7 @@ class CmdDockerClient(ContainerClient):
             if "No such container" in to_str(e.stdout):
                 raise NoSuchContainer(container_name)
             raise ContainerException(
-                "Docker process returned with errorcode %s" % e.returncode, e.stdout, e.stderr
+                f"Docker process returned with errorcode {e.returncode}", e.stdout, e.stderr
             )
 
     def copy_from_container(
@@ -219,6 +237,49 @@ class CmdDockerClient(ContainerClient):
                 "Docker process returned with errorcode %s" % e.returncode, e.stdout, e.stderr
             )
 
+    def push_image(self, docker_image: str) -> None:
+        cmd = self._docker_cmd()
+        cmd += ["push", docker_image]
+        LOG.debug("Pushing image with cmd: %s", cmd)
+        try:
+            safe_run(cmd)
+        except subprocess.CalledProcessError as e:
+            if "is denied" in to_str(e.stdout):
+                raise AccessDenied(docker_image)
+            if "does not exist" in to_str(e.stdout):
+                raise NoSuchImage(docker_image)
+            if "connection refused" in to_str(e.stdout):
+                raise RegistryConnectionError(e.stdout)
+            raise ContainerException(
+                f"Docker process returned with errorcode {e.returncode}", e.stdout, e.stderr
+            ) from e
+
+    def build_image(self, dockerfile_path: str, image_name: str, context_path: str = None):
+        cmd = self._docker_cmd()
+        dockerfile_path = Util.resolve_dockerfile_path(dockerfile_path)
+        context_path = context_path or os.path.dirname(dockerfile_path)
+        cmd += ["build", "-t", image_name, "-f", dockerfile_path, context_path]
+        LOG.debug("Building Docker image: %s", cmd)
+        try:
+            safe_run(cmd)
+        except subprocess.CalledProcessError as e:
+            raise ContainerException(
+                f"Docker build process returned with error code {e.returncode}", e.stdout, e.stderr
+            ) from e
+
+    def tag_image(self, source_ref: str, target_name: str) -> None:
+        cmd = self._docker_cmd()
+        cmd += ["tag", source_ref, target_name]
+        LOG.debug("Tagging Docker image %s as %s", source_ref, target_name)
+        try:
+            safe_run(cmd)
+        except subprocess.CalledProcessError as e:
+            if "No such image" in to_str(e.stdout):
+                raise NoSuchImage(source_ref)
+            raise ContainerException(
+                f"Docker process returned with error code {e.returncode}", e.stdout, e.stderr
+            ) from e
+
     def get_docker_image_names(self, strip_latest=True, include_tags=True):
         format_string = "{{.Repository}}:{{.Tag}}" if include_tags else "{{.Repository}}"
         cmd = self._docker_cmd()
@@ -479,7 +540,8 @@ class CmdDockerClient(ContainerClient):
         ports: Optional[PortMappings] = None,
         env_vars: Optional[Dict[str, str]] = None,
         user: Optional[str] = None,
-        cap_add: Optional[str] = None,
+        cap_add: Optional[List[str]] = None,
+        cap_drop: Optional[List[str]] = None,
         network: Optional[str] = None,
         dns: Optional[str] = None,
         additional_flags: Optional[str] = None,
@@ -513,7 +575,9 @@ class CmdDockerClient(ContainerClient):
         if user:
             cmd += ["--user", user]
         if cap_add:
-            cmd += ["--cap-add", cap_add]
+            cmd += list(itertools.chain.from_iterable(["--cap-add", cap] for cap in cap_add))
+        if cap_drop:
+            cmd += list(itertools.chain.from_iterable(["--cap-drop", cap] for cap in cap_drop))
         if network:
             cmd += ["--network", network]
         if dns:
